@@ -26,6 +26,14 @@
 #include "ADM_audiocodec/ADM_audiocodec.h"
 #include "ADM_codec.h"
 #include "DIA_coreToolkit.h"
+#include "ADM_vidMisc.h"
+
+#if 1
+#define aprintf printf
+#else
+#define aprintf(...) {}
+#endif
+
 ADM_EditorSegment::ADM_EditorSegment(void)
 {
 }
@@ -256,7 +264,7 @@ bool ADM_EditorSegment::deleteAll (void)
 bool        ADM_EditorSegment::undo(void)
 {
     if(undoSegments.empty()) return false;
-
+    clipboard.clear();
     segments=undoSegments.back(); 
     undoSegments.pop_back();
     updateStartTime();
@@ -337,7 +345,10 @@ int         ADM_EditorSegment::getNbSegments(void)
 bool         ADM_EditorSegment::updateStartTime(void)
 {
     int n=segments.size();
+    if(!n) return true;
+    
     uint64_t t=0;
+    t=segments[0]._startTimeUs;
     for(int i=0;i<n;i++)
     {
         segments[i]._startTimeUs=t;
@@ -436,6 +447,13 @@ bool        ADM_EditorSegment::getRefFromTime(uint64_t xtime,uint32_t *refVideo)
 */
 bool        ADM_EditorSegment::convertLinearTimeToSeg(  uint64_t frameTime, uint32_t *seg, uint64_t *segTime)
 {
+    if(!frameTime && segments.size()) // pick the first one
+    {
+        ADM_info("Frame time=0, taking first segment \n");
+        *seg=0;
+        *segTime=0; // ??
+        return true;
+    }
     for(int i=0;i<segments.size();i++)
     {
         if(segments[i]._startTimeUs<=frameTime && segments[i]._startTimeUs+segments[i]._durationUs>frameTime)
@@ -514,9 +532,11 @@ uint32_t    ADM_EditorSegment::intraTimeToFrame(uint32_t refVideo,uint64_t seekT
             ADM_error("Cannot find frame with time %"PRIu64"ms\n",seekTime/1000);
             ADM_assert(0);
         }
-        if(!(flags & AVI_KEY_FRAME))
+        uint32_t next;
+        v->_aviheader->getFlags(frame+1,&next);
+        if(!((next | flags) & AVI_KEY_FRAME)) // The 2nd field might be keyframe
         {
-                ADM_warning("Seeking to a non keyframe (time=%s)\n",ADM_us2plain(seekTime));
+                ADM_warning("Seeking to a non keyframe (time=%s), flags=%x, flagsNext=%x\n",ADM_us2plain(seekTime),flags,next);
                 ADM_warning("This is not normal unless you start frame is not a keyframe\n");
         }
         return frame;
@@ -714,7 +734,9 @@ bool        ADM_EditorSegment::removeEmptySegments(void)
         for(int i=0;i<n;i++)
         {
                 _SEGMENT *seg=getSegment(i);
+                _VIDEOS  *vid=this->getRefVideo(seg->_reference);
                 if(seg->_durationUs==0) index=i;
+                if(seg->_refStartTimeUs==0 && seg->_durationUs==vid->firstFramePts) index=i; 
         }
         if(index==-1) break;
         segments.erase(segments.begin()+index);
@@ -757,4 +779,90 @@ bool        ADM_EditorSegment::LinearToRefTime(int segNo,uint64_t linear,uint64_
     *refTime=(uint64_t )time;
     return true;
 }
+/**
+ * \fn copyToClipBoard
+ * \Brief copy the section between startTime and endTime to clipboard
+ * @param startTime
+ * @param endTime
+ * @return 
+ */
+bool        ADM_EditorSegment::copyToClipBoard(uint64_t startTime, uint64_t endTime)
+{
+    ADM_info("Copy to clipboard from %s",ADM_us2plain(startTime));
+    ADM_info("to %s\n",ADM_us2plain(endTime));
+    uint32_t startSeg,endSeg;
+    uint64_t startSegTime,endSegTime;
+    convertLinearTimeToSeg(  startTime, &startSeg,&startSegTime);
+    convertLinearTimeToSeg(  endTime, &endSeg,&endSegTime);
+    clipboard.clear();
+    for(int seg=startSeg;seg<=endSeg;seg++)
+    {
+        _SEGMENT s=segments[seg];
+        aprintf("Adding segment %d to clipboard\n",seg);
+        if(s._startTimeUs<=startTime && (s._startTimeUs+s._durationUs)>startTime)
+        {
+            // need to refine 1st seg
+            aprintf("Marker A is here\n");
+            uint64_t offset=startTime-s._startTimeUs;
+            s._refStartTimeUs+=offset;
+            s._durationUs-=offset;         // take into account the part we chopped
+        }
+        if(s._startTimeUs<=endTime && (s._startTimeUs+s._durationUs)>endTime)
+        {
+            aprintf("Marker B is here\n");
+            // need to refine last seg            
+            uint64_t offset=endTime-s._startTimeUs;
+            s._durationUs=offset;
+        }
+        // TODO refine timing for 1st/last/duration/...
+        clipboard.push_back(s);        
+    }
+    return false;
+}
+/**
+ * \fn pasteFromClipBoard
+ * \brief instert clipboard at currentTime position
+ * @param currentTime
+ * @return 
+ */
+bool        ADM_EditorSegment::pasteFromClipBoard(uint64_t currentTime)
+{
+    ADM_info("Pasting from clipboard to %s\n",ADM_us2plain(currentTime));
+    uint32_t startSeg;
+    uint64_t startSegTime;
+    convertLinearTimeToSeg(  currentTime, &startSeg,&startSegTime);    
+    ListOfSegments newSegs;
+    int n=segments.size();
+    for(int i=0;i<n;i++)
+    {
+        _SEGMENT s=segments[i];
+        if(i==startSeg)
+        {
+            // insert clipboard
+            // Do we need to split it ?
+            if(currentTime==s._startTimeUs)
+            {
+                // nope
+                for(int j=0;j<clipboard.size();j++) newSegs.push_back(clipboard[j]);
+            }else
+            {
+                 _SEGMENT pre=s,post=s;
+                 uint64_t offset=currentTime-s._startTimeUs;
+                 pre._durationUs=offset;
+                 post._refStartTimeUs+=offset;
+                 post._durationUs-=offset;
+                 newSegs.push_back(pre);
+                 for(int j=0;j<clipboard.size();j++) newSegs.push_back(clipboard[j]);
+                 newSegs.push_back(post);
+                 continue;
+            }
+                    
+        }
+        newSegs.push_back(s);
+    }
+    segments=newSegs;
+    updateStartTime();
+    return true;
+}
+
 //EOF
